(**
 dns_helper.mli --- Helpers to obtain relevant DNS information in a high level
 fashion.

 This API is part of ODNS library.
                                                                         
 Copyright (C)  2011  Jehan Hysseo <hysseo at zemarmot point net>
                                                                         
 This program is free software; you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 2 as          
 published by the Free Software Foundation.                              
                                                                         
 This program is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranty of            
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     
 General Public License for more details.                                

*)

(** This module provides functions to query Domain Name servers. It makes
 assumption to simplify the developer's task.
 In particular queries will be of the Internet (["IN"]) class, and the
timeout or retry values are the one by default.
The goal is to keep these functions very simple to use, though they still do
sometimes complex computations (like SRV), some verifications or multiple DNS
queries.

 If you need more control over the
 whole querying process, you might want to have a look to the
 [Dns.query] resolver class, or more generally to the {!module: Dns}
 module.
 *)

(**
  [srv_lookup ~query] will return an {b (ip, port)} list, for the requested
  [query]. This list will be in a ready to try-and-connect order.
  In other words, the developer should not reorder this list and should try to
  connect to the returned (ip, port) combinations in the given order.

  {e Implementation Note:}
  The function will first lookup for SRV queries on the name server.
  - If there are records for the service, then the IPs (both IPv4 and IPv6)
of the targets will be requested as well, and the whole list will be returned in
an order semi-randomly generated according to SRV rules. 
  - If there is no such SRV records, but the original target nevertheless has
  recorded Internet Addresses, those will be returned along with the default
  known port for the service. If the service has no known port, the IPs will
  still be returned, but with a [-1] port.
  This allows a developper to not be blocked if he works with
  services unknown on a user's system, or simply with experimental/custom
  services not assigned by IANA.

  @param query the query in the srv fashion, which is in the form
  ["_service._protocol.domain_name"].
  @return a list of {b (ip, port)} .
 @see <http://tools.ietf.org/html/rfc2782> RFC 2782: DNS SRV
 *)
val srv_lookup: query:string -> (string * int) list

(**
  [a_lookup domain] will make necessary queries to return IPv4 addresses for a
  given [domain].
  @param target is a domain whose IPv4 addresses one looks for.
  @return a list of such IPv4 addresses associated to this domain.
 *)
val a_lookup: target:string -> string list

(**
  [aaaa_lookup domain] will return the IPv6 addresses associated to [domain].
  @param target is a domain whose IPv6 addresses one looks for.
  @return a list of such IPv6 addresses.
 *)
val aaaa_lookup: target:string -> string list

(**
[address_lookup domain] requests both IPv4 and IPv6 addresses of [domain].
(basically {!a_lookup} and {!aaaa_lookup} concatanated).
@param target is a domain whose addresses one looks for.
@return a list of such addresses (can be IPv4, IPv6 or both).
 *)
val address_lookup: target:string -> string list

(*
TODO
[mx_lookup domain] lookups IP addresses to exchange emails.
@param target is a domain whose mail server one looks for.
@return a list of such addresses (can be IPv4, IPv6 or both), already ordered
according to server preferences.
@see <http://tools.ietf.org/html/rfc5321#section-5> RFC 5321: Simple Mail
Transfer Protocol (section 5 -- Address Resolution)
@see <http://tools.ietf.org/html/draft-daboo-srv-email-05> can also be an
interesting alternative for using SRV.
 *)
(* val mx_lookup: string -> string list*)

(* @param path where to save the cache.
 * By default try to store in /var/cache/odns (which should therefore be set
 * with the right rights, maybe by setting write rights on a group where the
 * executable is running.
 * If not possible, in $HOME/.odns/cache/.
 * If none of this is possible and no init has been set, then there won't be
 * cache system.
 (*val init: string -> unit*)
 *)

(* TODO: what about a generic configuration function to have more powerful
 * control (for instance on timeout or name servers) on all these while still
 * keeping them simple to use, and uncluttered with many options?
 *)
