(**
 host.ml --- "ring", the host-like command in Ocaml
                                                                         
 Copyright (C) 2002  Guillaume Valadon <guillaume@valadon.net>
                     Manu Letellier <manu.letellier@gmail.com>
               2010-2011  Jehan Hysseo <hysseo at zemarmot point net>
                                                                         
 This program is free software; you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 2 as          
 published by the Free Software Foundation.                              
                                                                         
 This program is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranty of            
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     
 General Public License for more details.                                

*)

(* Attempt to build a host command with Dns module *)

exception Timeout;;

(* type that represents 'host' configuration *)
type config =
{
   typ : string;
   hostserv : string list;
   wait : bool ref;  
   cls : string;
   recurs : bool ref;
   lst : bool ref
}
;;

(* This function will output answer the 'host' way *)
(* TODO: must accept several queries (though mainly unused, in theory, there can
 * be several queries in the same message) *)
let print_query_answer answ qtype host =
    let answered = ref false in
    let rec aux answers =
       match answers with
       | a::l -> 
          (if qtype = 255 (* all records *)|| a#_type = qtype
          (* TODO actually some kind of query type can imply several significant
           * response types. *)
          then
              answered := true;
              print_string (Dnsprint.pretty_record a)
              );
              aux l
       | [] ->
               if not !answered
               then
                   Printf.printf "%s has no %s record\n" host
                       (String.uppercase (Dnsprint.type_to_string qtype))
    in aux answ
;;


let usage =
   "ring: usage error (missing host argument)\n \
   Usage: ring [--help] [-lrw] [-t querytype] [-c class] host [server]...\n";;

(* Let's parse the command line *)
let parse_args = 
    let default = ref
    {
        typ = "*";
        hostserv = [];
        wait = ref false;
        cls = "IN";
        recurs = ref false;
        lst = ref false;
    } 
   and set_type def s =
       def := { !def with
       typ = String.uppercase s}
   and set_cls def x =
       def := { !def with cls = x}
   and anon def s =
       def := { !def with hostserv = !def.hostserv@[s]}
              in
              let lst = 
                  [("-c", Arg.String (set_cls default), "specifies the query\
                    class (default to Internet data: 'IN')");
                  ("-l", Arg.Set !default.lst, "turns on 'list mode'");
                  ("-r", Arg.Set !default.recurs, "disables recursive query");
                  ("-t", Arg.String (set_type default), "specifies the query\
                    type (default to any: '*')");
                  ("-w", Arg.Set !default.wait, "allows waiting forever until\
                  reply")] 
    in
    Arg.parse lst (anon default) usage;
    !default
;;


let rec classic set_server host conf search =
    let dns = new Dns.query in
    dns#set_nameservers set_server;
    let pkt = dns#raw_query in
    (* -r : Do a recursive request ?  *)
    if !(conf.recurs)
    then
        pkt#header#set_rd false
    else
        pkt#header#set_flags 256;

    (* Build the query using -c and -t *)
    if !(conf.lst)
    then
        dns#add_query ~qname:host ~qtype:"*" ~qclass:conf.cls
    else
        dns#add_query ~qname:host ~qtype:conf.typ ~qclass:conf.cls;

    (* wait forever *)
    if !(conf.wait)
    then
        dns#set_timeout (-1.);

        (* TODO: check if recursion desired really works. It doesn't look like. *)
        (*dns#raw_query#header#set_rd true;*)
    (* do the job *)
    dns#action;
    (* finally print the answer *)
    let answer =
        match dns#raw_answer with
        | Some a ->
                a
        | None ->
                raise Timeout
    in
    match answer#header#rcode with
    | 0 ->
            print_query_answer answer#answers (List.hd
            dns#raw_query#questions)#qtype host; (*255 = ANY *)
            (* TODO: I should be able to set several kind of qtype. *)
    | 1 ->
            print_string "Format error - The name server was\
            unable to interpret the query.\n"
    | 2 ->
            print_string "Server failure -\
            The name server was unable to process this query due\
            to a problem with the name server.\n"
    | 3 -> 
            if search != []
            then
                classic set_server (host ^ "." ^ (List.hd search)) conf (List.tl search)
            else
                Printf.printf "Host %s not found.\n" host
    | 4 ->
            print_string "Not Implemented -\
            The name server does not support the requested kind of query.\n"
    | 5 ->
            print_string "Refused -\
            The name server refuses to perform the specified operation\
            for policy reasons.\n"
    | other -> (* 6 - 15 reserved for future use. *)
            Printf.printf "An unexpected error number '%d' occurred.\n" other
;;


let axfr set_server host =
    let axfr = new Dns.dns_axfr in
    axfr#set_nameservers set_server;
   axfr#ls host;
   axfr#action;
   let lst = axfr#get_lst in
   match (List.hd lst)#header#rcode with
   | 0 -> 
      List.iter (fun x ->
	 print_query_answer x#answers 255 host) lst; 
   | 5 -> 
      print_string "Server failed: Query refused\n"
   | other -> 
      print_string ("TODO: error number "^(string_of_int other)^".")
;;

try
    let conf = parse_args in
    let host = List.hd conf.hostserv in
    let rec keep_uniques = function
        | [] -> []
        | a::[] -> [a]
        | a::l ->
                a::(keep_uniques (List.filter (fun x -> x != a) l))
    in
    let nameservers =
        let raw_ns =
            if List.length conf.hostserv = 2
            then
                begin
                    let srv = List.nth conf.hostserv 1 in
                    Printf.printf "Using domain name server: %s\n" srv;
                    flush stdout;
                    [srv]
                end
            else if List.length conf.hostserv > 2
            then
                begin
                    let srvs = List.tl conf.hostserv in
                    Printf.printf "Using domain name servers in this order: %s\n"
                        (List.fold_left (fun s1 s2 -> s1 ^ ", " ^ s2)
                            (List.hd srvs) (List.tl srvs));
                    flush stdout;
                    srvs
                end
            else
                []
        in
        keep_uniques (List.flatten (List.map Tools.getaddr raw_ns))
    in
    (* Let's use command line arguments *)
    if !(conf.lst)
    then
        axfr nameservers host
    else
        classic nameservers host conf [] (*search*)
with
| Failure "hd" ->
        print_string usage;
        exit 1
| Dns.Unknown_type str ->
        print_string ("Unknown query type: " ^ str ^ "\n");
        exit 2
| Dns.Unknown_class str ->
        print_string ("Unknown query class: " ^ str ^ "\n");
        exit 2
| Tools.Invalid_IP_or_DN dn ->
        Printf.printf "The provided name server (%s) is not a valid IP \
        or domain name.\n" dn;
        exit 1
| Timeout ->
        print_string "Query timeout - \
        No answer has been received from any name server.\n"
;;
